
package com.taobao.idlefish.flutterboost.Cache;

import java.util.HashMap;
import java.util.Map;
import java.util.Stack;

public class StackCache {

    private static StackCache Instance = new StackCache();

    private int inMemoryCount = 2;

    private Stack<String> keyStack = new Stack<>();
    private Map<String, Class> typesMap = new HashMap<>();
    private Map<String, StackCacheIObject> inMemoryObjectsMap = new HashMap<>();
    private Map<String, Boolean> loadingMap = new HashMap<>();
    private String cacheDirPath;

    public static StackCache GetInstance() {
        return Instance;
    }

    public int getInMemoryCount() {
        return inMemoryCount;
    }

    public void setInMemoryCount(int inMemoryCount) {
        this.inMemoryCount = inMemoryCount;
    }

    public String getCacheDirPath() {
        return cacheDirPath;
    }

    public void setCacheDirPath(String cacheDirPath) {
        this.cacheDirPath = cacheDirPath;
    }

    public void put(StackCacheIObject object, String key) {

        if (object == null || key == null) return;

        object.setKey(key);

        typesMap.put(key, object.getClass());

        if(!keyStack.contains(key)) keyStack.push(key);

        inMemoryObjectsMap.put(key, object);

        for (int i = keyStack.size() - inMemoryObjectsMap.size();
             i < keyStack.size() && inMemoryObjectsMap.size() > inMemoryCount; i++) {

            String keyToSave = keyStack.get(i);
            if (inMemoryObjectsMap.containsKey(keyToSave)) {

                StackCacheIObject ob = inMemoryObjectsMap.get(keyToSave);
                inMemoryObjectsMap.remove(keyToSave);
                ob.writeToFile(flutterCacheDir(), new StackCacheObjectCallback() {
                    @Override
                    public void onFinished(Error error, String path) {
                        //TODO:
                    }
                });
            }

        }

    }

    public StackCacheIObject top() {

        if (!keyStack.isEmpty()) {
            return inMemoryObjectsMap.get(keyStack.peek());
        }

        return null;
    }

    public StackCacheIObject get(String key) {
        if (key != null) {
            return inMemoryObjectsMap.get(key);
        }
        return null;
    }


    public void remove(String key) {

        if(key == null) return;

        if (!inMemoryObjectsMap.isEmpty()) {
            inMemoryObjectsMap.remove(key);
        }


        if (!typesMap.isEmpty()) {
            typesMap.remove(key);
        }

        if (!keyStack.isEmpty()) {
            keyStack.remove(key);
        }

        preloadIfNeeded();

    }

    public boolean empty() {
        return keyStack.isEmpty();
    }


    public void clear() {
        keyStack.clear();
        inMemoryObjectsMap.clear();
        typesMap.clear();
    }

    private String flutterCacheDir() {
        return cacheDirPath + "/FlutterScreenShots";
    }

    private void preloadIfNeeded() {

        if (!keyStack.isEmpty()) return;

        for (int i = 0; i < keyStack.size(); i++) {
            final String key = keyStack.get(i);
            Class typeClass = typesMap.get(key);
            StackCacheIObject oldCache = inMemoryObjectsMap.get(key);
            if (typeClass != null && oldCache == null) {
                try {
                    final StackCacheIObject cache = (StackCacheIObject) typeClass.newInstance();
                    loadingMap.put(key, true);
                    cache.setKey(key);
                    cache.restoreFromFile(flutterCacheDir(), new StackCacheObjectCallback() {
                        @Override
                        public void onFinished(Error error, String path) {
                            loadingMap.remove(key);
                            if (typesMap.containsKey(key)) {
                                inMemoryObjectsMap.put(key, cache);
                            }
                        }
                    });
                } catch (InstantiationException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }


        }

    }


}
